#!/usr/bin/bash
# config_ipa_client.sh - Config an IPA client and test NFS related utilities

Usage() {
cat <<END
Usage: config_ipa_client.sh <-i IPA Server IP |--cleanup> [--config_krb]

        -h|--help                        # Print this help

        [Basic Function: Config current client as an IPA Domain Member / Leave current IPA Domain]
        -i|--ipa_server_ip <IP address>  # Specify IP of an IPA Server for target AD DS Domain
        -c|--cleanup                     # Leave IPA Domain and delete entry in IPA database

        [Extra Functions: Config extra roles (secure NFS Client..) after integration]
        --config_krb                     # Config current client as a Secure NFS client
END
}


infoecho() { echo -e "\n${P:-DO ==>}" "\E[1;34m" "$@" "\E[0m"; }
errecho()  { echo -e "\n${P:-DO ==>}" "\E[31m" "$@" "\E[0m"; }


# PART: [Dependency] Specify all dependencies during integration related jobs

# Specify packages for "IPA Domain Integration"
export pkgs="bind-utils ipa-client ipa-admintools ntpdate"

# Specify modified configuration files or keytab
# Conf files modified by ipa-client-install will restore automatically after uninstallation
export HOSTNAME_CONF=/etc/hostname
export RESOLV_CONF=/etc/resolv.conf
export KRB5_KEYTAB=/etc/krb5.keytab

# Specify target IPA Domain and its Domain LDAP Server information
export IPA_SERVER_IP=""
export IPA_SERVER_FQDN=""
export IPA_DOMAIN_NAME=""
export IPA_DOMAIN_SUPERPW_DEFAULT='fsqe2015!'
export IPA_DOMAIN_SUPERPW=$IPA_DOMAIN_SUPERPW_DEFAULT
export IPA_CLIENT_SHORTNAME=""
export IPA_CLIENT_FQDN=""

# Specify functions
export cleanup=no;
export config_krb=no;

# PART: [Preparing] Prepare IPA Domain / Server Information

[ $# -eq 0 ] && {
       Usage;
       exit 1;
}

while [ -n "$1" ]; do
        case "$1" in
        -i|--ipa_server_ip) IPA_SERVER_IP="$2";shift 2;;
        -c|--cleanup) cleanup="yes";shift 1;;
        -p|--passwd)  IPA_DOMAIN_SUPERPW="$2";shift 2;;
        -h|--help)    Usage;exit 0;;
        --config_krb) config_krb="yes";shift 1;;
        *) break;;
        esac
done

infoecho "Obtain information of the IPA Domain and its Domain Controller..."

if [ "$cleanup" == "yes" ]; then
        echo "{Info} Try to leave the current IPA Domain"
        ipa host-del $(hostname)
        ipa dnsrecord-del $(hostname -d) $(hostname|cut -d'.' -f-1) &>/dev/null <<< yes
        if ipa-client-install --uninstall &>/dev/null <<<no ; then
                infoecho "Leave IPA Domain or cleanup Success..."
                exit 0;
        else
                errecho "Leave IPA Domain or cleanup Failed..."
                exit 1;
        fi
fi

rpm -q bind-utils || yum install -y bind-utils
IPA_SERVER_FQDN=$(nslookup $IPA_SERVER_IP $IPA_SERVER_IP | awk '/name =/{print $NF}' | sed 's/.$//g')
IPA_DOMAIN_NAME=$(cut -d'.' -f2- <<< $IPA_SERVER_FQDN)
IPA_DOMAIN_SUPERPW_DEFAULT='fsqe2015!'
IPA_DOMAIN_SUPERPW=$IPA_DOMAIN_SUPERPW_DEFAULT
IPA_CLIENT_SHORTNAME=$(hostname|cut -d'.' -f-1)
IPA_CLIENT_FQDN=${IPA_CLIENT_SHORTNAME}.${IPA_DOMAIN_NAME}

[ -z "$IPA_SERVER_IP" ] && {
       echo "{WARN} Please specify IP of target IPA Domain's Domain Server"
       Usage;
       exit 1;
}

infoecho "Check connections with IPA Server by IP..."
ping -c 2 "$IPA_SERVER_IP" || {
       echo "{WARN} Can not connect to IPA Server via IP: '${IPA_SERVER_IP}'"
       exit 1;
}

if [ $((${#IPA_SERVER_FQDN}*${#IPA_DOMAIN_NAME}*${#IPA_DOMAIN_SUPERPW})) -eq 0 ]; then
        echo "{WARN} Can not get sufficient IPA Domain information, please check IPA Server";
        exit 1
else
        echo "{Info} Will start to integrate into IPA Domain: $IPA_DOMAIN_NAME";
fi

# PART: [Basic Function] Config current client as an IPA Domain Member

infoecho "1. Close the firewall and block NetworkManager..."
[ -f /etc/init.d/iptables ] && service iptables stop
which systemctl &>/dev/null && systemctl stop firewalld

# Block NetworkManager to ensure change in resolv.conf is effective
service NetworkManager status &>/dev/null && service NetworkManager stop &>/dev/null
service NetworkManager status &>/dev/null || service network start &>/dev/null

infoecho "2. Make sure necessary packages are installed..."
rpm -q $pkgs || yum -y install $pkgs

infoecho "3. Change hostname to integrate into IPA DNS Domain..."
sysctl kernel.hostname=$IPA_CLIENT_FQDN
echo $IPA_CLIENT_FQDN > $HOSTNAME_CONF
hostname $IPA_CLIENT_FQDN

infoecho "4. Change DNS Server to IPA Domain DNS..."
echo -e "search $IPA_DOMAIN_NAME\nnameserver $IPA_SERVER_IP" > $RESOLV_CONF
cat $RESOLV_CONF

infoecho "5. Sync time with IPA Server..."
service ntpd status && service ntpd stop
ntpdate -4 $IPA_SERVER_IP

infoecho "6. Integrate into IPA Domain..."
ipa-client-install --force-join --unattended --domain $IPA_DOMAIN_NAME --server $IPA_SERVER_FQDN \
	--principal admin --password $IPA_DOMAIN_SUPERPW

infoecho "7. Login as domain administrator and add principals for secure NFS tests"
kinit admin <<< $IPA_DOMAIN_SUPERPW
klist

# Add A Records for current client to enable DNS functions in IPA Domain
ipa dnsrecord-del $IPA_DOMAIN_NAME $IPA_CLIENT_SHORTNAME <<< yes
ipa dnsrecord-add $IPA_DOMAIN_NAME $IPA_CLIENT_SHORTNAME --a-rec=$(getDefaultIp);
if ipa host-find $IPA_CLIENT_FQDN || ipa host-add $IPA_CLIENT_FQDN; then
        infoecho "Configure IPA Client complete..."
else
        errecho "Configure IPA Client Failed, cannot add A record in database"
fi

# PART: [Extra Functions] Config current client as a Secure NFS client

if [ "$config_krb" == "yes" ]; then
        infoecho "krb 1. Add principals for current IPA client..."

        # Add and fetch root/ and nfs/ principals
        for serviceType in root nfs; do
            # Add principal by domain super user admin
            ipa service-add $serviceType/$(hostname)
            # Fetch principal from remote server to local keytab
            ipa-getkeytab -s $IPA_SERVER_FQDN -p $serviceType/$(hostname) -k /etc/krb5.keytab
            # Check whether local IPA client could read target principal from keytab
            ipa service-show $serviceType/$(hostname)

            if ! klist -kte | grep root; then
                echo "{WARN} Failed to add and fetch $(serviceType)/$(hostname) principal"
                exit 1;
            fi
        done

        infoecho "Configure Secure NFS Client complete..."
fi
